#pragma once 

#include <iostream>
#include <sstream>
#include <complex>
#include <cstdint>
#include <cstring>
#include <string>
using namespace std;
#include <getopt.h>

typedef uint8_t u8;
typedef uint32_t u32;

template<class T>
void set_from_range(T& var, T value, T lo, T hi, string label) {

	if (!((lo <= value) && (value <= hi))) { 

		ostringstream oss; 
		oss << label << " must be set between " << lo << " and " << hi << endl;  
		throw std::invalid_argument( oss.str() ); 

	} else { 

		var = value;

	}
}

class Terminal {
public:
	// line types
	static string line_type_1;
	static string line_type_2;
	static string line_type_3;
	static string line_type_4;
	static string line_type_5;

	// begin underline
	static string b_underline;
	// end underline
	static string e_underline;
	// begin bold
	static string b_bold;
	// end bold
	static string e_bold;

	/**
	 * Convert string to bold for terminal display
	 */
	static string bold( string s );
	
	/**
	 * Convert string to underline for terminal display
	 */
	static string underline( string s );
	
};

/* ==================================================================
 * SUBPROGRAM
 *
 *  void help_clopt_wa( char _shortf, string _longf, 
 *		string arg, string text );
 *
 * WHAT
 * 
 *  Display help about Command Line OPTion With Argument
 *
 * PARAMETERS
 *
 *	@paramdef(char, _shortf) : command line option in short format
 *	@paramdef(string, _longf) : command line option in long format
 *	@paramdef(string, arg) : argument of the option
 *	@paramdef(string, text) : description
 *
 * ==================================================================
 */
void help_clopt_wa( char _shortf, string _longf, string arg, string text );

/* ==================================================================
 * SUBPROGRAM
 *
 *  void help_clopt_na( char _shortf, string _longf, 
 *		string arg, string text );
 *
 * WHAT
 * 
 *  Display help about Command Line OPTion with No Argument
 *
 * PARAMETERS
 *
 *	@paramdef(char, _shortf) : command line option in short format
 *	@paramdef(string, _longf) : command line option in long format
 *	@paramdef(string, text) : description
 *
 * ==================================================================
 */
void help_clopt_na( char _shortf, string _longf, string text );



